#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include "ams.h"

main (int argc, char *argv[])
{

    AMS_Comm alice;
    char **mem_list, mem_name[255], fld_name[255], new_val[255], *msg;
    int err, i = 0;

    extern int PrintMemory(AMS_Comm, char *);
    extern int PrintField(AMS_Comm, char *, char *);
    extern int ModifyField(AMS_Comm, char *, char *, char *);
    extern int RecvUpdate(AMS_Comm, char *);

    /* Attach to a Communicator */
    err = AMS_Comm_attach("my_first_publisher", &alice);
    AMS_Check_error(err, &msg);

    /* Get the memory list */
    err = AMS_Comm_get_memory_list(alice, &mem_list);
    AMS_Check_error(err, &msg);

    while (mem_list[i])
        printf("%s\n", mem_list[i++]);

    /* Print the content of a memory strucure */
    printf(" Which memory do you want to view ?"); gets(mem_name);
    PrintMemory(alice, mem_name);


    /* Print the content of field */
    printf(" Which field do you want to view ?"); gets(fld_name);
    PrintField(alice, mem_name, fld_name);

    /* Change the field value and send the update */
    printf(" Which field do you want to modify ?"); gets(fld_name);
    printf(" \n Enter the new value:"); gets(new_val);
    ModifyField(alice, mem_name, fld_name, new_val);

    /* Receive an update from server */
    printf("Receiving a memory update \n");
    RecvUpdate(alice, mem_name);
    PrintField(alice, mem_name, fld_name);

    getchar();

    return 0;
}
        
int PrintField(AMS_Comm alice, char *mem_name, char *fld_name)
{
    AMS_Memory mem;
    AMS_Memory_type mtype;
    AMS_Data_type dtype;
    AMS_Shared_type stype;
    AMS_Reduction_type rtype;
    int err, len;
    void *addr;
    char *msg;

    char buf[255], val[255];

    /* Get the memory id: Attach the memory */
    err = AMS_Memory_attach(alice, mem_name, &mem);
    AMS_Check_error(err, &msg);

    /* Get Field infos */
    err = AMS_Memory_get_field_info(mem, fld_name, &addr, &len, &dtype, &mtype, &stype, &rtype);
    AMS_Check_error(err, &msg);
    printf("Field %s descriptions: \n", fld_name);
        
    /* Field's Memory type */
    switch(mtype) {
    case AMS_READ:
        strcpy(buf, "Read");
        break;
    case AMS_WRITE:
        strcpy(buf, "Read/Write");
        break;
    default:
        strcpy(buf, "Undefined");
        break;
    }
    printf("\t Memory type: %s \n", buf);

    /* Data type and data format */
    switch(dtype) {
    case AMS_INT:
        strcpy(buf, "Interger");
        sprintf(val, "%d", *((int *) addr));
        break;
    case AMS_DOUBLE:
        strcpy(buf, "Double");
        sprintf(val, "%f", *((int *)addr));
        break;
    default:
        strcpy(buf, "Undefined");
        break;
    }
    printf("\t Data type: %s \n", buf);
    printf("\t Data value: %s \n", val);
    /* Shared type and Reduction type */
    switch(stype) {
    case AMS_COMMON:
        strcpy(buf, "COMMON");
        break;
    case AMS_DISTRIBUTED:
        strcpy(buf, "Distributed");
        break;
    case AMS_REDUCED:
        switch(rtype) {
        case AMS_MIN:
            strcpy(buf, "Reduced: MIN");
            break;
        case AMS_MAX:
            strcpy(buf, "Reduced: MAX");
            break;
        case AMS_SUM:
            strcpy(buf, "Reduced: SUM");
            break;
        default:
            strcpy(buf, "Reduced: Undefined");
            break;
        }
        break;
    default:
        strcpy(buf, "Undefined");
        break;
    }
    printf("\t Shared type: %s \n", buf);

    /* Let's Deattach the memory for testing */
    err = AMS_Memory_deattach(mem);
    AMS_Check_error(err, &msg);

    return AMS_ERROR_NONE;
}

int ModifyField(AMS_Comm alice, char *mem_name, char *fld_name, char *new_val)
{
    AMS_Memory mem;
    AMS_Memory_type mtype;
    AMS_Data_type dtype;
    AMS_Shared_type stype;
    AMS_Reduction_type rtype;
    int AMS_int;
    double AMS_dbl;
    int err, len;
    void *addr;
    char *msg;

    /* Get the memory id: Attach the memory */
    err = AMS_Memory_attach(alice, mem_name, &mem);
    AMS_Check_error(err, &msg);

    /* Get Field infos */
    err = AMS_Memory_get_field_info(mem, fld_name, &addr, &len, &dtype, &mtype, &stype, &rtype);
    AMS_Check_error(err, &msg);
        
    /* Field's Memory type */
    switch(mtype) {
    case AMS_READ:
        printf(" Field is read only \n");
        return AMS_ERROR_NONE;;
    case AMS_WRITE:
        break;
    default:
        printf("Undefined memory type \n");
        return AMS_ERROR_NONE;;
    }
    /* Data type and data format */
    switch(dtype) {
    case AMS_INT:
        AMS_int = atoi(new_val);
        addr = &AMS_int;
        len = sizeof(int);
        break;
    case AMS_DOUBLE:
        AMS_dbl =  atof(new_val);
        addr = &AMS_dbl;
        len = sizeof(double);
        break;
    default:
        printf("undefined AMS_data_type\n");
        break;
    }

    err = AMS_Memory_set_field_info(mem, fld_name, addr, len);
    AMS_Check_error(err, &msg);

    err = AMS_Memory_update_send_begin(mem);
    AMS_Check_error(err, &msg);
        
    return 0;
}

int PrintMemory(AMS_Comm alice, char *mem_name)
{
    char **fld_list, *msg;
    AMS_Memory mem;
    int i = 0, err;

    /* Attach the memory structure */
    err = AMS_Memory_attach(alice, mem_name, &mem);
    AMS_Check_error(err, &msg);

    err = AMS_Memory_get_field_list(mem, &fld_list);
    AMS_Check_error(err, &msg);

    printf("Memory %s contains the following fields\n", mem_name);
    i = 0;
    while(fld_list[i])
        printf("\t\t %s\n", fld_list[i++]);
    /* Frees the memory created within get_field_list */
    free(fld_list);

    return 0;
}
        
int RecvUpdate(AMS_Comm alice, char *mem_name)
{
    AMS_Memory mem;
    int i = 0, err;
    char *msg;

    /* Attach the memory structure */
    err = AMS_Memory_attach(alice, mem_name, &mem);
    AMS_Check_error(err, &msg);

    /* Receive an update */
    err = AMS_Memory_update_recv_begin(mem);
    AMS_Check_error(err, &msg);

    return 0;
}
